//
//  DDAppDelegate.m
//  BrokenConnection
//
//  Created by Damien DeVille on 04/12/2011.
//  Copyright (c) 2011 Damien DeVille. All rights reserved.
//

#import "DDAppDelegate.h"
#import "DDDownloadOperation.h"

@interface DDAppDelegate ()

@property (nonatomic,strong) NSOperationQueue *queue;
@property (nonatomic,strong) DDDownloadOperation *downloadOperation;

- (void)createConnectionAndAddToQueue;
- (void)createConnectionAndStartOnMainQueue;
- (void)createConnectionAndStartOnGlobalQueue;

@end


@implementation DDAppDelegate

@synthesize window = _window;
@synthesize queue = _queue;
@synthesize downloadOperation = _downloadOperation;

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
	UIWindow *window = [[UIWindow alloc] initWithFrame: [[UIScreen mainScreen] bounds]];
	[self setWindow: window];
	
	UIViewController *viewController = [[UIViewController alloc] init];
	[[self window] setRootViewController: viewController];
	[[self window] makeKeyAndVisible];
	
	[self setQueue: [[NSOperationQueue alloc] init]];
	
	NSURL *remoteURL = [NSURL URLWithString: @"http://apple.com"];
	DDDownloadOperation *downloadOperation = [[DDDownloadOperation alloc] initWithRemoteURL: remoteURL];
	[self setDownloadOperation: downloadOperation];
	
	void (^completionBlock) () = ^ {
		NSURLResponse *response = nil;
		NSError *error = nil;
		NSData *data = [[self downloadOperation] responseBlock](&response, &error);
		
		if (data == nil)
			NSLog(@"Oh Noooo: %@", error);
		else
			NSLog(@"Cool %@", response);
	};
	
	[[self downloadOperation] setCompletionBlock: completionBlock];
	
	// Fails
	[self performSelector: @selector(createConnectionAndAddToQueue) withObject: nil afterDelay: 0.0f];
	
	// Works (but is pointless)
//	[self performSelector: @selector(createConnectionAndStartOnMainQueue) withObject: nil afterDelay: 0.0f];
	
	// Fails
//	[self performSelector: @selector(createConnectionAndStartOnGlobalQueue) withObject: nil afterDelay: 0.0f];
	
	return YES;
}

- (void)createConnectionAndAddToQueue
{
	[[self queue] addOperation: [self downloadOperation]];
}

- (void)createConnectionAndStartOnMainQueue
{
	[[self downloadOperation] start];
}

- (void)createConnectionAndStartOnGlobalQueue
{
	dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
		[[self downloadOperation] start];
	});
}

@end
