//
//  DDDownloadOperation.m
//  BrokenConnection
//
//  Created by Damien DeVille on 04/12/2011.
//  Copyright (c) 2011 Damien DeVille. All rights reserved.
//

#import "DDDownloadOperation.h"

static NSString * const DDDownloadOperationIsExecutingKey = @"isExecuting";
static NSString * const DDDownloadOperationIsFinishedKey = @"isFinished";

@interface DDDownloadOperation () <NSURLConnectionDelegate>

@property (strong) NSURL *remoteURL;
@property BOOL isExecuting, isFinished;
@property (strong) NSOperationQueue *delegateQueue;
@property (strong) NSURLConnection *connection;
@property (strong) NSURLResponse *response;
@property (strong) NSMutableData *data;
@property (copy) DDDownloadOperationResponse responseBlock;

- (void)_finish;

@end


@implementation DDDownloadOperation

@synthesize remoteURL = _remoteURL;
@synthesize isExecuting = _isExecuting, isFinished = _isFinished;
@synthesize delegateQueue = _delegateQueue;
@synthesize connection = _connection;
@synthesize response = _response;
@synthesize data = _data;
@synthesize responseBlock = _responseBlock;

- (id)initWithRemoteURL:(NSURL *)remoteURL
{
	self = [self init];
	if (self == nil)
		return nil;
	
	_isExecuting = NO, _isFinished = NO;
	
	_remoteURL = remoteURL;
	
	_delegateQueue = [[NSOperationQueue alloc] init];
	[_delegateQueue setMaxConcurrentOperationCount: 1];
	[_delegateQueue setSuspended: YES];
	
	return self;
}

- (BOOL)isConcurrent
{
	return YES;
}

- (void)start
{
	NSLog(@"Starting");
	
	[self willChangeValueForKey: DDDownloadOperationIsExecutingKey];
	[self setIsExecuting: YES];
	[self didChangeValueForKey: DDDownloadOperationIsExecutingKey];
	
	[[self delegateQueue] setSuspended: NO];
	
	NSURLRequest *request = [[NSURLRequest alloc] initWithURL: _remoteURL];
	NSURLConnection *connection = [[NSURLConnection alloc] initWithRequest: request delegate: self startImmediately: NO];
	[self setConnection: connection];
	[[self connection] setDelegateQueue: [self delegateQueue]];
	
	[[self connection] start];
}

- (void)cancel
{
	[[self connection] cancel];
	[[self delegateQueue] cancelAllOperations];
	
	[super cancel];
}

- (void)_finish
{
	[self willChangeValueForKey: DDDownloadOperationIsExecutingKey];
	[self setIsExecuting: NO];
	[self didChangeValueForKey: DDDownloadOperationIsExecutingKey];
	
	[self willChangeValueForKey: DDDownloadOperationIsFinishedKey];
	[self setIsFinished: YES];
	[self didChangeValueForKey: DDDownloadOperationIsFinishedKey];
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
	NSLog(@"Received response");
	
	[self setResponse: response];
	[self setData: [NSMutableData data]];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
	NSLog(@"Finished loading");
	
	NSURLResponse *response = [self response];
	NSData *data = [self data];
	
	[self setResponseBlock: ^ NSData * (NSURLResponse **responseRef, NSError **errorRef) {
		if (responseRef != NULL)
			*responseRef = response;
		return data;
	}];
	
	[self _finish];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
	NSLog(@"Failed");
	
	NSURLResponse *response = [self response];
	
	[self setResponseBlock: ^ NSData * (NSURLResponse **responseRef, NSError **errorRef) {
		if (responseRef != NULL)
			*responseRef = response;
		if (errorRef != NULL)
			*errorRef = error;
		return nil;
	}];
	
	[self _finish];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
	[[self data] appendData: data];
}

@end
